from .base_test import BaseTest

SAMPLE_DATASOURCE_PARAMS = {
    "description": "Python Integration Test Data Source. Delete if seen.",
    "title": "Python Integration Test Data Source",
    "source": "api",
    "config": '{"type":"json","method":"GET","transform":"products","url":"https://dummyjson.com/products","headers":"{}"}',
}


class TestDatasources(BaseTest):

    def setUp(self):
        super().setUp()
        self.sampleDatasourceId = self.onvoSDK.datasources.create(
            SAMPLE_DATASOURCE_PARAMS
        )[
            "id"
        ]  # Skipping test create

    def test_list(self):
        self.assertShouldRaise(None, self.onvoSDK.datasources.list)

    def test_get(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.datasources.get(self.sampleDatasourceId)
        )

    def test_get_data(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.datasources.get_data(self.sampleDatasourceId)
        )

    def test_update(self):
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.datasources.update(
                self.sampleDatasourceId, {"title": "Renaming Test"}
            ),
        )

    def test_populate_columns(self):
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.datasources.populate_columns(self.sampleDatasourceId),
        )

    def tearDown(self) -> None:
        self.onvoSDK.datasources.delete(self.sampleDatasourceId)  # Skipping test delete



from onvo.base_resource import BaseResource


class Accounts(BaseResource):
    def list(self):
        return self.base_get("/accounts")

    def get(self, id):
        return self.base_get(f"/accounts/{id}")



from .base_test import BaseTest


class TestAccounts(BaseTest):

    def test_list(self):
        self.assertShouldRaise(None, self.onvoSDK.accounts.list)

    def test_get(self):
        sample_id = self.onvoSDK.accounts.list()[0]["id"]
        self.assertShouldRaise(None, lambda: self.onvoSDK.accounts.get(sample_id))



from onvo.base_resource import BaseResource


class Dashboards(BaseResource):
    def list(self):
        return self.base_get("/dashboards")

    def get(self, id):
        return self.base_get(f"/dashboards/{id}")

    def create(self, data):
        return self.base_put(f"/dashboards", data=data)

    def update(self, id, data):
        return self.base_post(f"/dashboards/{id}", data=data)

    def delete(self, id):
        return self.base_delete(f"/dashboards/{id}")



from onvo.base_resource import BaseResource


class Automations(BaseResource):
    def list(self):
        return self.base_get("/automations")

    def get(self, id):
        return self.base_get(f"/automations/{id}")

    def create(self, data):
        return self.base_put(f"/automations", data=data)

    def update(self, id, data):
        return self.base_post(f"/automations/{id}", data=data)

    def delete(self, id):
        return self.base_delete(f"/automations/{id}")



from onvo.base_resource import BaseResource


class Widgets(BaseResource):
    def list(self, dashboard_id):
        return self.base_get("/widgets", params={"dashboard": dashboard_id})

    def get(self, id):
        return self.base_get(f"/widgets/{id}")

    def get_image(self, id):
        return self.base_get(f"/widgets/{id}/image")

    # TODO: Fix breakage in create widgets
    # def create(self, dashboard_id, query):
    #     return self.base_put(
    #         "/widgets", params={"query": query}, data={"dashboard": dashboard_id}
    #     )

    # def update(self, id, data):
    #     return self.base_post(f"/widgets/{id}", data=data)

    # def delete(self, id):
    #     return self.base_delete(f"/widgets/{id}")



from onvo.base_resource import BaseResource


class Questions(BaseResource):
    def list(self, dashboard_id: str):
        return self.base_get("/questions", params={"dashboard": dashboard_id})

    # TODO: Fix breakage in ask question
    # def delete(self, id: str):
    #     return self.base_delete(f"/questions/{id}")

    # def create(self, dashboard_id: str, query: str):
    #     return self.base_post(
    #         "/questions", params={"query": query, "dashboard": dashboard_id}
    #     )



from .base_test import BaseTest

SAMPLE_EMBED_USER_PARAMS = {
    "id": "sample-embed-user-id",
    "name": "Python Integration Test User",
    "email": "test@test.com",
    "metadata": {"hello": "world"},
}


class TestEmbedUsers(BaseTest):

    def setUp(self):
        super().setUp()
        self.sampleEmbedUserId = self.onvoSDK.embed_users.upsert(
            SAMPLE_EMBED_USER_PARAMS
        )[
            "id"
        ]  # Skipping test create

    def test_list(self):
        self.assertShouldRaise(None, self.onvoSDK.embed_users.list)

    def test_get(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.embed_users.get(self.sampleEmbedUserId)
        )

    def test_get_access_token(self):
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.embed_users.get_access_token(self.sampleEmbedUserId),
        )

    def test_upsert_update(self):
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.embed_users.upsert(
                {"id": self.sampleEmbedUserId, "name": "Renamed Python Test User"},
            ),
        )

    def tearDown(self) -> None:
        self.onvoSDK.embed_users.delete(self.sampleEmbedUserId)  # Skipping test delete



from .base_test import BaseTest

SAMPLE_DASHBOARD_PARAMS = {
    "description": "Sample dashboard for widget testing",
    "title": "Widget Testing Dashboard",
}

SAMPLE_AUTOMATION_PARAMS = {
    "created_by": "197302e5-88e2-49f4-bbe7-92b5a4dc4264",
    "description": "A sample description",
    "email_format": "This is an automation from Onvo",
    "email_subject": "This is an automation from Onvo",
    "enabled": "false",
    "output_format": "link",
    "recipient_type": "internal",
    "schedule": "",
    "title": "API datasource test",
}


class TestAutomations(BaseTest):

    def setUp(self):
        super().setUp()
        self.sampleDashboardId = self.onvoSDK.dashboards.create(
            SAMPLE_DASHBOARD_PARAMS
        )["id"]
        # Skipping test create
        self.sampleAutomationId = self.onvoSDK.automations.create(
            {**SAMPLE_AUTOMATION_PARAMS, "dashboard": self.sampleDashboardId}
        )["id"]

    def test_list(self):
        self.assertShouldRaise(None, self.onvoSDK.automations.list)

    def test_get(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.automations.get(self.sampleAutomationId)
        )

    def test_update(self):
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.automations.update(
                self.sampleAutomationId,
                {"description": "A New Test Description. Delete if seen."},
            ),
        )

    def tearDown(self) -> None:
        self.onvoSDK.dashboards.delete(self.sampleDashboardId)
        self.onvoSDK.automations.delete(self.sampleAutomationId)  # Skipping test delete



import os
from onvo.resources.accounts import Accounts
from onvo.resources.automations import Automations
from onvo.resources.dashboards import Dashboards
from onvo.resources.datasources import Datasources
from onvo.resources.embed_users import EmbedUsers
from onvo.resources.teams import Teams
from onvo.resources.sessions import Sessions
from onvo.resources.questions import Questions
from onvo.resources.widgets import Widgets

from onvo.resources.dashboard import Dashboard

try:
    default_endpoint = os.environ["ONVO_API_ENDPOINT"]
    default_api_key = os.environ["ONVO_API_KEY"]
except:
    default_endpoint = None
    default_api_key = None


class Onvo:
    def __init__(self, endpoint=default_endpoint, api_key=default_api_key):
        self.endpoint = endpoint
        self.api_key = api_key
        self.check_init_params()

        params = [endpoint, api_key]

        self.accounts = Accounts(*params)
        self.automations = Automations(*params)
        self.datasources = Datasources(*params)
        self.dashboards = Dashboards(*params)
        self.embed_users = EmbedUsers(*params)
        self.teams = Teams(*params)
        self.sessions = Sessions(*params)
        self.questions = Questions(*params)
        self.widgets = Widgets(*params)

    def dashboard(self, dashboard_id):
        return Dashboard(dashboard_id, self.endpoint, self.api_key)

    def check_init_params(self):
        if self.endpoint is None:
            raise Exception(
                "Onvo object has no endpoint! Initalize with the endpoint parameter or use ONVO_API_ENDPOINT"
            )
        if self.api_key is None:
            raise Exception(
                "Onvo object has no api key! Initalize with the api_key parameter or use ONVO_API_KEY"
            )



from onvo.resources.dashboard_resources.datasources import DashboardDatasources


class Dashboard:
    def __init__(self, dashboard_id: str, endpoint: str, api_key: str):
        params = [dashboard_id, endpoint, api_key]

        self.datasources = DashboardDatasources(*params)



from onvo.base_resource import BaseResource


class Datasources(BaseResource):
    def list(self):
        return self.base_get("/datasources")

    def get(self, id):
        return self.base_get(f"/datasources/{id}")

    def get_data(self, id):
        return self.base_get(f"/datasources/{id}/data")

    def populate_columns(self, id):
        return self.base_post(f"/datasources/{id}/populate-columns")

    def create(self, data):
        return self.base_put(f"/datasources", data=data)

    def update(self, id, data):
        return self.base_post(f"/datasources/{id}", data=data)

    def delete(self, id):
        return self.base_delete(f"/datasources/{id}")



from onvo.base_resource import BaseResource


class Sessions(BaseResource):
    def list(self, dashboard_id: str):
        return self.base_get("/sessions", params={"dashboard": dashboard_id})

    def delete(self, dashboard_id: str):
        return self.base_delete("/sessions", params={"dashboard": dashboard_id})

    def upsert(self, dashboard_id: str, user_id: str):
        session_data = self.base_post(
            "/sessions", data={"dashboard": dashboard_id, "user": user_id}
        )
        return {**session_data, "url": f"{self.endpoint}{session_data['url']}"}



from onvo.resources.dashboard_resources.base_resource import DashboardBaseResource


class DashboardDatasources(DashboardBaseResource):
    def list(self):
        return self.base_get(f"/dashboards/{self.dashboard_id}/datasources")

    def link(self, datasource_id):
        return self.base_put(
            f"/dashboards/{self.dashboard_id}/datasources",
            data={"datasourceId": datasource_id},
        )

    def unlink(self, datasource_id):
        return self.base_delete(
            f"/dashboards/{self.dashboard_id}/datasources/{datasource_id}"
        )



from .base_test import BaseTest

SAMPLE_DASHBOARD_PARAMS = {
    "description": "Test Description.",
    "title": "Python Integration Test Dashboard. Delete if seen.",
}


class TestDashboards(BaseTest):

    def setUp(self):
        super().setUp()
        self.sampleDashboardId = self.onvoSDK.dashboards.create(
            SAMPLE_DASHBOARD_PARAMS
        )[
            "id"
        ]  # Skipping test create

    def test_list(self):
        self.assertShouldRaise(None, self.onvoSDK.dashboards.list)

    def test_get(self):
        self.assertShouldRaise(
            None, lambda: self.onvoSDK.dashboards.get(self.sampleDashboardId)
        )

    def test_update(self):
        self.assertShouldRaise(
            None,
            lambda: self.onvoSDK.dashboards.update(
                self.sampleDashboardId,
                {"description": "A New Test Description. Delete if seen."},
            ),
        )

    def tearDown(self) -> None:
        self.onvoSDK.dashboards.delete(self.sampleDashboardId)  # Skipping test delete



import requests
from requests.utils import quote
import json


class BaseResource:
    def __init__(self, endpoint, api_key):
        self.endpoint = endpoint
        self.headers = {"x-api-key": api_key, "Content-Type": "application/json"}

    def handle_response(self, response):
        response.raise_for_status()
        try:
            return response.json()
        except ValueError:
            return response.text()

    def process_options(self, options):
        if "data" in options:
            options["data"] = json.dumps(options["data"])
        # Standardizing params to use '%20' instead of '+' to replace spaces for all SDK requests
        if "params" in options:
            options["params"] = "&".join(
                [f"{key}={quote(value)}" for key, value in options["params"].items()]
            )

    # TODO : Add documentation marking 'params' as the query & 'data' as the body
    def base_request(self, handler, subdirectory, additional_options):
        url = f"{self.endpoint}{subdirectory}"
        options = {"headers": self.headers, **additional_options}
        self.process_options(options)

        response = handler(url, **options)
        return self.handle_response(response)

    def base_get(self, subdirectory, **options):
        return self.base_request(requests.get, subdirectory, options)

    def base_put(self, subdirectory, **options):
        return self.base_request(requests.put, subdirectory, options)

    def base_post(self, subdirectory, **options):
        return self.base_request(requests.post, subdirectory, options)

    def base_delete(self, subdirectory, **options):
        return self.base_request(requests.delete, subdirectory, options)



from onvo.base_resource import BaseResource


class DashboardBaseResource(BaseResource):
    def __init__(self, dashboard_id, endpoint, api_key):
        super().__init__(endpoint, api_key)
        self.dashboard_id = dashboard_id
